//#######################################################################
//#
//#	Copyright 	Earth People Technology Inc. 2013
//#
//#
//# File Name:  EPT_570_AP_U2_Top.v
//# Author:     Earth People Technology
//# Date:       March 27, 2013
//# Revision:   A
//#
//# Development: EPT Data Collection Project 
//# Application: Altera MAX II CPLD
//# Description: This file contains verilog code which will allow access
//#              to Active Transfer Library. 
//#               
//#              
//#
//#************************************************************************
//#
//# Revision History:	
//#			DATE		VERSION		DETAILS		
//#			3/27/13 	1			Created
//#
//#									
//#
//#######################################################################


//************************************************************************
//* Module Declaration
//************************************************************************

module EPT_570_AP_U2_Top (

	
	input  wire [1:0]          aa,
	input  wire [1:0]          bc_in,
	output wire [2:0]          bc_out,
	inout  wire [7:0]		   bd_inout,
	
	input wire   [1:0]          LB_SER,       //XIOH -- J10
	input wire   [5:0]          LB_AD,        //AD   -- J9
	input wire   [7:0]          LB_IOH,       //XIOH -- J10
	input wire   [7:0]          LB_IOL,       //XIOL -- J8
	
	
	//Transceiver Control Signals
	output wire                TR_DIR_1,
	output reg                 TR_OE_1,
	
	output wire                TR_DIR_2,
	output reg                 TR_OE_2,

	output wire                TR_DIR_3,
	output reg                 TR_OE_3,

	
	input wire                SW_USER_1,
	input wire                SW_USER_2,

	output wire [3:0]          LED
	);

   //-----------------------------------------------
   // Control Register State Machine Parameters
   //-----------------------------------------------

   //Header Bytes for the Transfer Loopback detection
   parameter                   TRANSFER_CONTROL_BYTE1 = 8'h5A;
   parameter                   TRANSFER_CONTROL_BYTE2 = 8'hC3;
   parameter                   TRANSFER_CONTROL_BYTE3 = 8'h7E;
   
   //State Machine Transfer Loopback detection
   parameter                   TRANSFER_CONTROL_IDLE = 0,
			                   TRANSFER_CONTROL_HDR1 = 1,
			                   TRANSFER_CONTROL_HDR2 = 2,
			                   TRANSFER_DECODE_BYTE   = 3,
			                   TRANSFER_CONTROL_SET  = 4;

   
   //----------------------------------------------
   // USB Transfer State Machine Parameters  
   //----------------------------------------------
	parameter 	IDLE							= 0,
				START_TRANSFER 					= 1,
				FIRST_BYTE_EN       			= 2,
				FIRST_BYTE_RDY      			= 3,
				SECOND_BYTE_EN					= 4,
				SECOND_BYTE_RDY 				= 5;
 
   //----------------------------------------------
   // Global Reset Parameters  
   //----------------------------------------------
  parameter                   GLOBAL_RESET_COUNT = 12'h09c8;
 
//***************************************************************************
//* Internal Signals and Registers Declarations
//***************************************************************************
    wire                       CLK_66;
	wire                       RST;

	wire [23:0]                UC_IN;
	wire [21:0]                UC_OUT;
	
	//Finite State Machine control registers
	reg 	[5:0] 					state, next;

	//LED registers
	reg                        led_reset;
	
	//Switch registers
	reg                        switch_reset;
	
	//Transfer registers
	reg                        transfer_out_reg;
	wire                       transfer_in_received;
	wire [7:0]                 transfer_in_byte;
	wire  [7:0]                transfer_out_byte;
	reg  [3:0]                 transfer_to_host_counter;
	reg  [3:0]                 transfer_to_host_state;

    //Transfer Control registers
	reg                        transfer_in_received_reg;
	reg  [3:0]                 transfer_control_state;
	reg  [7:0]                 transfer_control_byte;
	
    //Transfer Write from Arduino
	reg                        transfer_write_reg;
	reg                        transfer_write;
	reg  [9:0]                 transfer_write_data;
	reg  [7:0]                 analog_monitor_lower_byte;
	reg  [1:0]                 analog_monitor_upper_byte;
	reg  [2:0]                 transfer_address;
	reg  [2:0]                 analog_monitor_address;
	reg                        analog_monitor_en;

	//Reset signals
	wire                       reset;
	reg [11:0]                 reset_counter;
	reg                        reset_signal_reg;
	
	//Input/Output Signals
	reg                        start_stop_cntrl;
	
	//Control Signals for Active Transfer Endterms
	reg  [5:0]                 start_transfer_array;
	
	
    reg  [5:0]                 transfer_out;
	wire [5:0]                 transfer_busy_array;
	
	//Registers to control the order of byte
	//written to the PC
	reg                        first_byte_complete;
	reg                        second_byte_complete;

`ifdef SIM
   reg [8*26:1] state_name;
`endif
	


//***************************************************************************
//* 	Signal Assignments	
//***************************************************************************
  assign             TR_DIR_1  = 1'b1; //1 = A to B; 0 = B to A

   assign            TR_DIR_2  = 1'b1; //1 = A to B; 0 = B to A

   assign            TR_DIR_3  = 1'b1; //1 = A to B; 0 = B to A

   //Clock and Reset
   assign            CLK_66 = aa[1];
   assign            RST = reset;
   assign            reset = reset_signal_reg;
   
   //Transfer registers
   assign            transfer_out_byte = (state[START_TRANSFER] | state[FIRST_BYTE_EN] | 
                     state[FIRST_BYTE_RDY] ? {6'h0,transfer_write_data[9:8]} : (state[SECOND_BYTE_EN] | 
					 state[SECOND_BYTE_RDY]) ? transfer_write_data[7:0] : 8'hz);
   
   //LED[3] is used to signify to the  user that the Start 
   //switch is enabled
   assign             LED[3] =  ~(transfer_out) ? 1'b0 : 1'bz;  
   
   assign            LED[2:0] = ~transfer_address;
     
//**************************************************
//* 	Reset Signal	
//**************************************************
	always @(posedge CLK_66 or negedge aa[0])
	begin
	  if(!aa[0])
	  begin
			reset_signal_reg <= 1'b0;
			reset_counter <= 0;
	  end
	  else
	  begin
		if( reset_counter < GLOBAL_RESET_COUNT )
		begin
			reset_signal_reg <= 1'b0;
			reset_counter <= reset_counter + 1'b1;
		end
		else
		begin
			reset_signal_reg <= 1'b1;
		end
	  end
	end

   //-----------------------------------------------
   // Register the inputs
   //-----------------------------------------------
  always @(posedge CLK_66 or negedge RST)
  begin
     if(!RST)
	 begin
	     analog_monitor_en <= 1'b0;
	     analog_monitor_lower_byte <= 0;
		 analog_monitor_upper_byte <= 0;
         analog_monitor_address <= 0;
	 end
	else 
	begin
	     analog_monitor_en <= LB_IOH[0];
	     analog_monitor_lower_byte <= LB_IOL;
		 analog_monitor_upper_byte <= {LB_IOH[4],LB_SER[1]};
         analog_monitor_address <= LB_IOH[3:1];
	end
  end

   //-----------------------------------------------
   // Detect Transfer From Arduino using the enable:
   // LB_IOH[0]
   //-----------------------------------------------
    always @(posedge CLK_66 or negedge RST) 
	begin
       if (!RST)
       begin
			transfer_write_reg <= 1'b0;
			transfer_write <= 1'b0;
			transfer_write_data <= 0;
            transfer_address <= 0;
            TR_OE_1  = 1'b1;
            TR_OE_2  = 1'b1;
            TR_OE_3  = 1'b1;  
       end 
       else 
       begin 
	        if(start_stop_cntrl)
			begin
                 TR_OE_1  = 1'b0;
                 TR_OE_2  = 1'b0;
                 TR_OE_3  = 1'b0;  
	             if(analog_monitor_en & !transfer_write_reg)
			     begin
			         transfer_write_reg <= 1'b1;
			     	transfer_write <= 1'b1;
			     	transfer_address <= analog_monitor_address;
			     	transfer_write_data <= {analog_monitor_upper_byte,analog_monitor_lower_byte};
			     end
	             else if(analog_monitor_en & transfer_write_reg)
			     begin
			         transfer_write_reg <= 1'b1;
			     	transfer_write <= 1'b0;
			     end
	             else if(!analog_monitor_en & transfer_write_reg)
			     begin
			         transfer_write_reg <= 1'b0;
			     	transfer_write <= 1'b0;
			     end
			end
			else
			begin
                 TR_OE_1  = 1'b0;
                 TR_OE_2  = 1'b1;
                 TR_OE_3  = 1'b1;  
			end
	   end
    end
	
   //-----------------------------------------------
   // Select the Transfer EndTerm using the address
   // from the Arduino
   //-----------------------------------------------
    always @(posedge CLK_66 or negedge RST) 
	begin
	    if(!RST)
		begin
		     transfer_out <= 0;
		end
		else 
		begin
		     if(state[START_TRANSFER])
			 begin
		         case(transfer_address)
			     3'h0:
			     begin
		              transfer_out <= 0;
			     end
			    3'h1:
			     begin
		              transfer_out <= 6'h01;
			     end
			     3'h2:
			     begin
		              transfer_out <= 6'h02;
			     end
			     3'h3:
			     begin
		              transfer_out <= 6'h04;
			     end
			     3'h4:
			     begin
		              transfer_out <= 6'h08;
			     end
			     3'h5:
			     begin
		              transfer_out <= 6'h10;
			     end
			     3'h6:
			     begin
		              transfer_out <= 6'h20;
			     end
			     default:
			     begin
		              transfer_out <= 6'h00;
			     end
			     endcase
			 end
		end
	end

   //-----------------------------------------------
   // Transfer Upper/Lower byte using the selected Active 
   // Transfer EndTerm. Byte transferred is determined
   // by the USB Transfer State machine
   //-----------------------------------------------
    always @(posedge CLK_66 or negedge RST) 
	begin
       if (!RST)
       begin
			first_byte_complete <= 1'b0;
			second_byte_complete <= 1'b0;
			start_transfer_array <= 0;
       end 
       else 
       begin 
	        if(state[FIRST_BYTE_EN] & !first_byte_complete)
			begin
			    first_byte_complete <= 1'b1;
				start_transfer_array <= transfer_out;
			end
	        else if(state[FIRST_BYTE_EN] & first_byte_complete)
			begin
			    first_byte_complete <= 1'b1;
			    start_transfer_array <= 0;
			end
	        else if(!state[FIRST_BYTE_EN] & first_byte_complete)
			begin
			    first_byte_complete <= 1'b0;
			end

	        if(state[SECOND_BYTE_EN] & !second_byte_complete)
			begin
			    second_byte_complete <= 1'b1;
				start_transfer_array <= transfer_out;
			end
	        else if(state[SECOND_BYTE_EN] & second_byte_complete)
			begin
			    second_byte_complete <= 1'b1;
			    start_transfer_array <= 0;
			end
	        else if(!state[SECOND_BYTE_EN] & second_byte_complete)
			begin
			    second_byte_complete <= 1'b0;
			end
			end
    end

   //-----------------------------------------------
   // State Machine: Control Register from Transfer In 
   //-----------------------------------------------
    always @(posedge CLK_66 or negedge RST) 
	begin
	     if (!RST)
		 begin
			 transfer_in_received_reg <= 1'b0;
			 transfer_control_state <= TRANSFER_CONTROL_IDLE;
			 led_reset <= 1'b0;
			 switch_reset <= 1'b0;
			 start_stop_cntrl <= 1'b0;
		 end 
		 else 
		 begin 
		     if(transfer_in_received & !transfer_in_received_reg)
			 begin
			     transfer_in_received_reg <= 1'b1;
		         case(transfer_control_state)
			     TRANSFER_CONTROL_IDLE:
		             if((transfer_in_byte == TRANSFER_CONTROL_BYTE1))
			             transfer_control_state <= TRANSFER_CONTROL_HDR1;
				     else if((transfer_in_byte != TRANSFER_CONTROL_BYTE1))
			             transfer_control_state <= TRANSFER_CONTROL_IDLE;
				     else
				         transfer_control_state <= TRANSFER_CONTROL_IDLE;
			     TRANSFER_CONTROL_HDR1:
		             if((transfer_in_byte == TRANSFER_CONTROL_BYTE2))
			             transfer_control_state <= TRANSFER_CONTROL_HDR2;
				     else if((transfer_in_byte != TRANSFER_CONTROL_BYTE2))
			             transfer_control_state <= TRANSFER_CONTROL_IDLE;
				     else
				         transfer_control_state <= TRANSFER_CONTROL_HDR1;
			     TRANSFER_CONTROL_HDR2:
		             if((transfer_in_byte == TRANSFER_CONTROL_BYTE3))
			             transfer_control_state <= TRANSFER_DECODE_BYTE;
				     else if((transfer_in_byte != TRANSFER_CONTROL_BYTE3))
			             transfer_control_state <= TRANSFER_CONTROL_IDLE;
				     else
				         transfer_control_state <= TRANSFER_CONTROL_HDR2;
			     TRANSFER_DECODE_BYTE:
			     begin
					 start_stop_cntrl <= transfer_in_byte[0];
					 led_reset <= transfer_in_byte[2];
					 switch_reset <= transfer_in_byte[3];
			         transfer_control_state <= TRANSFER_CONTROL_SET;
				 end
			     TRANSFER_CONTROL_SET:
			     begin
			         transfer_control_state <= TRANSFER_CONTROL_IDLE;
				 end
			     endcase
			 end
			 else if(!transfer_in_received & transfer_in_received_reg)
			     transfer_in_received_reg <= 1'b0;
         end
	end	 

   //-----------------------------------------------
   // State Machine: USB Transfer
   //-----------------------------------------------
   // Next State Logic
   always @(posedge CLK_66 or negedge RST)
     begin
	if (!RST)
	  begin
	     state <= 6'h000;
	     state[IDLE] <= 1'b1;
	  end
	else
	  state <= next;
     end

     // State Definitions
   always @ ( state or transfer_write or first_byte_complete or
		      second_byte_complete or transfer_write or start_transfer_array or
			  transfer_busy_array) 
     begin
	next = 6'h000;

	if (state[IDLE])
	  begin
	     if (transfer_write)
	       next[START_TRANSFER] = 1'b1;
	     else
	       next[IDLE] = 1'b1;
	  end
	
	if (state[START_TRANSFER])
			next[FIRST_BYTE_EN] = 1'b1;

	if (state[FIRST_BYTE_EN])
		if ( first_byte_complete )
			next[FIRST_BYTE_RDY] = 1'b1;
		else 
			next[FIRST_BYTE_EN] = 1'b1;

	if (state[FIRST_BYTE_RDY])
		if ( transfer_busy_array == 0)
			next[SECOND_BYTE_EN] = 1'b1;
		else 
		    next[FIRST_BYTE_RDY] = 1'b1;

	if (state[SECOND_BYTE_EN])
		if ( second_byte_complete )
			next[SECOND_BYTE_RDY] = 1'b1;
		else
			next[SECOND_BYTE_EN] = 1'b1;
	
	if (state[SECOND_BYTE_RDY])
		if ( transfer_busy_array == 0)
			next[IDLE] = 1'b1;
		else
			next[SECOND_BYTE_RDY] = 1'b1;


`ifdef SIM
 	   if ( state == ( 1 << IDLE ))
		   state_name = "IDLE";
	   else if ( state == ( 1 << START_TRANSFER ))
		   state_name = "START_TRANSFER";
	   else if ( state == ( 1 << FIRST_BYTE_EN ))
		   state_name = "FIRST_BYTE_EN";
	   else if ( state == ( 1 << FIRST_BYTE_RDY ))
		   state_name = "FIRST_BYTE_RDY";
	   else if ( state == ( 1 << SECOND_BYTE_EN ))
		   state_name = "SECOND_BYTE_EN";
	   else if ( state == ( 1 << SECOND_BYTE_RDY ))
		   state_name = "SECOND_BYTE_RDY";
`endif


	end//end of state machine

	
	//-----------------------------------------------
   // Instantiate the EPT Active Transfer Library
   //-----------------------------------------------

	active_transfer_library	   ACTIVE_TRANSFER_LIBRARY_INST
	(	
	.aa                        (aa),
	.bc_in                     (bc_in),
	.bc_out                    (bc_out),
	.bd_inout                  (bd_inout),

	.UC_IN                     (UC_IN),
	.UC_OUT                    (UC_OUT)
	
	);
	
   //-----------------------------------------------
   // Instantiate the EPT Active Modules
   //-----------------------------------------------
wire [22*6-1:0]  uc_out_m;
eptWireOR # (.N(6)) wireOR (UC_OUT, uc_out_m);
	active_transfer            ACTIVE_TRANSFER_INST_1
	(
	 .uc_clk                   (CLK_66),
	 .uc_reset                 (RST),
	 .uc_in                    (UC_IN),
	 .uc_out                   (uc_out_m[ 0*22 +: 22 ]),
	
	 .start_transfer           (start_transfer_array[0]),
	 .transfer_received        (),
	 
	 .transfer_busy            (transfer_busy_array[0]),
	
	 .uc_addr                  (3'h1),

	 .transfer_to_host         (transfer_out_byte),
	 .transfer_to_device       ()	
	);
	

	active_transfer            ACTIVE_TRANSFER_INST_2
	(
	 .uc_clk                   (CLK_66),
	 .uc_reset                 (RST),
	 .uc_in                    (UC_IN),
	 .uc_out                   (uc_out_m[ 1*22 +: 22 ]),
	
	 .start_transfer           (start_transfer_array[1]),
	 .transfer_received        (transfer_in_received),
	 
	 .transfer_busy            (transfer_busy_array[1]),
	
	 .uc_addr                  (3'h2),

	 .transfer_to_host         (transfer_out_byte),
	 .transfer_to_device       (transfer_in_byte)	
	);
	
	active_transfer            ACTIVE_TRANSFER_INST_3
	(
	 .uc_clk                   (CLK_66),
	 .uc_reset                 (RST),
	 .uc_in                    (UC_IN),
	 .uc_out                   (uc_out_m[ 2*22 +: 22 ]),
	
	 .start_transfer           (start_transfer_array[2]),
	 .transfer_received        (),
	 
	 .transfer_busy            (transfer_busy_array[2]),
	
	 .uc_addr                  (3'h3),

	 .transfer_to_host         (transfer_out_byte),
	 .transfer_to_device       ()	
	);

	active_transfer            ACTIVE_TRANSFER_INST_4
	(
	 .uc_clk                   (CLK_66),
	 .uc_reset                 (RST),
	 .uc_in                    (UC_IN),
	 .uc_out                   (uc_out_m[ 3*22 +: 22 ]),
	
	 .start_transfer           (start_transfer_array[3]),
	 .transfer_received        (),
	 
	 .transfer_busy            (transfer_busy_array[3]),
	
	 .uc_addr                  (3'h4),

	 .transfer_to_host         (transfer_out_byte),
	 .transfer_to_device       ()	
	 );

	active_transfer            ACTIVE_TRANSFER_INST_5
	(
	 .uc_clk                   (CLK_66),
	 .uc_reset                 (RST),
	 .uc_in                    (UC_IN),
	 .uc_out                   (uc_out_m[ 4*22 +: 22 ]),
	
	 .start_transfer           (start_transfer_array[4]),
	 .transfer_received        (),
	 
	 .transfer_busy            (transfer_busy_array[4]),
	
	 .uc_addr                  (3'h5),

	 .transfer_to_host         (transfer_out_byte),
	 .transfer_to_device       ()	
	);
	
	active_transfer            ACTIVE_TRANSFER_INST_6
	(
	 .uc_clk                   (CLK_66),
	 .uc_reset                 (RST),
	 .uc_in                    (UC_IN),
	 .uc_out                   (uc_out_m[ 5*22 +: 22 ]),
	
	 .start_transfer           (start_transfer_array[5]),
	 .transfer_received        (),
	 
	 .transfer_busy            (transfer_busy_array[5]),
	
	 .uc_addr                  (3'h6),

	 .transfer_to_host         (transfer_out_byte),
	 .transfer_to_device       ()	
	);
	
endmodule
